<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DTO\Response\Message;

use Teknisa\MAN\Utilities\Util;

class CampanhaCompreGanheProdutosController extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceManager;
    protected $utilService; 
    protected $crudService;
    protected $parameterBag;
    protected $campanhaService;
    
    public function __construct($crudService, $parameterBag, $paramBag, $dataSourceManager, Util $utilService, $campanhaService) {
        $this->crudService          = $crudService;
        $this->parameterBag         = $parameterBag;
        $this->dataSourceManager    = $dataSourceManager;
		$this->utilService          = $utilService;
		$this->campanhaService      = $campanhaService;
    }
        
    const dataSourceProdutos        = 'vnd17001_cadastroAutomaticoDeProdutosParticipantes';   
    const dataSourceSelectProdutos  = 'vnd17001_produtosParticipantes_produtos';    
    const dataSourceProdutoBrinde   = 'vnd17001_cadastroAutomaticoDeBrindes';
    const dataSourceSelectBrinde    = 'vnd17001_produtosParticipantes_Brinde';
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProdutos;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPCOMPGANHE', 'DTINIVGCAMPCG');
        parent::find($request,$response);
    }
    public function find2(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProdutoBrinde;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPCOMPGANHE', 'DTINIVGCAMPCG');
        parent::find($request,$response);
    }
    
    public function findBrindes(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceSelectBrinde;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPCOMPGANHE', 'DTINIVGCAMPCG');
        parent::find($request,$response);
    }
    
    public function findProdutos(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceSelectProdutos;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPCOMPGANHE', 'DTINIVGCAMPCG');
        parent::find($request,$response);
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceProdutos;
        $row = $request->getDataSet()->getRows()[0];
        
        $CDCAMPCOMPGANHE = $row['CDCAMPCOMPGANHE'];
        $DTINIVGCAMPCG   = $row['DTINIVGCAMPCG'];
        $NRORG           = $row['NRORG'];
        
        if($row['TIPOPROD'] == 'I') {
            $CDPRODUTOINI    = $row['CDPRODUTOINI'];
            $CDPRODUTOFIM    = $row['CDPRODUTOFIN'];

            $intervalo = $this->campanhaService->getIntervaloProdutos($CDCAMPCOMPGANHE, $DTINIVGCAMPCG, $NRORG, $CDPRODUTOINI, $CDPRODUTOFIM);
            
            $auxIntervalo = [];
            foreach($intervalo as $produto) {
                $auxIntervalo[] = $produto['CDPRODUTO'];
            }
            
            $intervalo = $auxIntervalo;
        } else {
            $intervalo = $row['PRODUTOS'];
        }
        
        try {
            
            for($i = 0; $i < count($intervalo); $i++) {
                $CDPRODUTO = $intervalo[$i];
                $push = $this->campanhaService->saveCampanhaProduto($CDCAMPCOMPGANHE, $DTINIVGCAMPCG, $CDPRODUTO, $NRORG);
            }
            $response->addDataSet(new DataSet($this->dataSourceName, []));
        }  catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    
    }
    
    public function delete(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceProdutos;
        parent::delete($request, $response);
    }
    
    public function saveBrindes(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceProdutoBrinde;
        $rows = $request->getDataSet()->getRows();
        
        $row = $rows[0];
        
        $CDCAMPCOMPGANHE = $row['CDCAMPCOMPGANHE'];
        $DTINIVGCAMPCG   = $row['DTINIVGCAMPCG'];
        $NRORG           = $row['NRORG'];
        
        if($row['TIPOPROD'] == 'I') {
            $CDPRODUTOINI    = $row['CDPRODUTOINI'];
            $CDPRODUTOFIM    = $row['CDPRODUTOFIN'];

            $intervalo = $this->campanhaService->getIntervaloBrindes($CDCAMPCOMPGANHE, $DTINIVGCAMPCG, $NRORG, $CDPRODUTOINI, $CDPRODUTOFIM);
            
            $auxIntervalo = [];
            foreach($intervalo as &$produto) {
                $auxIntervalo[] = $produto['CDPRODUTO'];
            }
            
            $intervalo = $auxIntervalo;
        } else {
            $intervalo = $row['PRODUTOS'];
        }
        
        try {
            
            for($i = 0; $i < count($intervalo); $i++) {
                $CDPRODUTO = $intervalo[$i];
                $this->campanhaService->saveCampanhaBrinde($CDCAMPCOMPGANHE, $DTINIVGCAMPCG, $CDPRODUTO, $NRORG);
            }
            
            $response->addDataSet(new DataSet($this->dataSourceName, []));
        }  catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    
    }
    
    public function deleteBrindes(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceProdutoBrinde;
        parent::delete($request, $response);
    }
}